//--------------------------------------------------------------------------------------
// This program demonstrates the parallax occlusion mapping technique as described
// in the article "A Closer Look At Parallax Occlusion Mapping" by Jason Zink.
//--------------------------------------------------------------------------------------
#include "DXUT.h"
#include "DXUTmisc.h"
#include "DXUTgui.h"
#include "DXUTsettingsDlg.h"
#include "SDKmisc.h"

#define DEG2RAD( a ) ( a * D3DX_PI / 180.f )


//--------------------------------------------------------------------------------------
// Structures
//--------------------------------------------------------------------------------------
struct SimpleVertex
{
    D3DXVECTOR3 Pos;  
    D3DXVECTOR2 Tex; 
	D3DXVECTOR3 Nrm;  
	D3DXVECTOR3 Tan;  
	D3DXVECTOR3 Bin;  
};


//--------------------------------------------------------------------------------------
// Global Variables
//--------------------------------------------------------------------------------------
CDXUTDialogResourceManager			g_DialogResourceManager;// manager for shared resources of dialogs
CD3DSettingsDlg						g_D3DSettingsDlg;       // Device settings dialog
CDXUTDialog			                g_HUD;                  // manages the 3D UI


ID3D10InputLayout*                  g_pVertexLayout = NULL; 
ID3D10Buffer*                       g_pVertexBuffer = NULL;
ID3D10Buffer*                       g_pIndexBuffer = NULL;

ID3D10Effect*                       g_pPOMEffect = NULL;      

ID3D10EffectTechnique*              g_pPOMTech = NULL;

ID3D10EffectMatrixVariable*         g_pWorldVar = NULL;
ID3D10EffectMatrixVariable*         g_pWorldViewProjVar = NULL;

ID3D10EffectVectorVariable*			g_pEyePosVar = NULL;
ID3D10EffectVectorVariable*			g_pLightPosVar = NULL;

ID3D10EffectShaderResourceVariable* g_pColorTexVar = NULL;
ID3D10EffectShaderResourceVariable* g_pHeightTexVar = NULL;
ID3D10ShaderResourceView*           g_pColorTexRV = NULL;
ID3D10ShaderResourceView*           g_pHeightTexRV = NULL;

ID3D10RenderTargetView*				g_pBackBufferView = NULL;
ID3D10DepthStencilView*				g_pBackBufferDepthView = NULL;

D3DXMATRIX                          g_World;
D3DXMATRIX                          g_View;
D3DXMATRIX                          g_Proj;
D3DXMATRIX                          g_WorldView;
D3DXMATRIX                          g_WorldViewProj;
D3DXVECTOR4							g_EyePosition;
D3DXVECTOR4							g_LightPosition;

//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
#define IDC_TOGGLEFULLSCREEN			1
#define IDC_TOGGLEREF					2
#define IDC_CHANGEDEVICE				3

//--------------------------------------------------------------------------------------
// Forward declarations 
//--------------------------------------------------------------------------------------
bool    CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BufferFormat, bool bWindowed, void* pUserContext );
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBufferSurfaceDesc, void* pUserContext );
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBufferSurfaceDesc, void* pUserContext );
void    CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext );
void    CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext );
void    CALLBACK OnD3D10DestroyDevice( void* pUserContext );
void    CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext );
void    CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );
void    CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext );
bool    CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext );


//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // Set DXUT callbacks
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );

    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );

    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen

    g_D3DSettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); int iY = 10; 
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );

    DXUTCreateWindow( L"Parallax Occlusion Mapping Demo" );
    DXUTCreateDevice( true, 640, 480 );
    DXUTMainLoop(); // Enter into the DXUT render loop

    return DXUTGetExitCode();
}


//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BufferFormat, bool bWindowed, void* pUserContext )
{
	if ( DeviceType == D3D10_DRIVER_TYPE_REFERENCE )
		return false;
	else
		return true;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC *pBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr = S_OK;

    V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );


    // Read the D3DX effect file
    DWORD dwShaderFlags = D3D10_SHADER_ENABLE_STRICTNESS;

	#if defined( DEBUG ) || defined( _DEBUG )
    // Set the D3D10_SHADER_DEBUG flag to embed debug information in the shaders.
    // Setting this flag improves the shader debugging experience, but still allows 
    // the shaders to be optimized and to run exactly the way they will run in 
    // the release configuration of this program.
    dwShaderFlags |= D3D10_SHADER_DEBUG;
    #endif
    
	ID3D10Blob*	pBlobErrors = NULL;

	hr = D3DX10CreateEffectFromFile( L"Parallax_Occlusion_Mapping.fx", NULL, NULL, "fx_4_0", dwShaderFlags, 
		0, pd3dDevice, NULL, NULL, &g_pPOMEffect, &pBlobErrors, NULL );
    
	if( FAILED( hr ) )
    {
		LPVOID pError = NULL;
		char* pErrorCharacters = NULL;

		if( pBlobErrors )
		{
			pError = pBlobErrors->GetBufferPointer();
			pErrorCharacters = (char*)pError;
		}

        MessageBox( NULL, L"The FX file cannot be located.  Please run this executable from the directory that contains the FX file.", L"Error", MB_OK );
        V_RETURN( hr );
    }


	// Read the needed parameters from the effect file
    g_pPOMTech = g_pPOMEffect->GetTechniqueByName( "Parallax_Occlusion_Mapping" );
	g_pWorldVar = g_pPOMEffect->GetVariableByName( "mW" )->AsMatrix();
	g_pWorldViewProjVar = g_pPOMEffect->GetVariableByName( "mWVP" )->AsMatrix();
	g_pEyePosVar = g_pPOMEffect->GetVariableByName( "EyePosition" )->AsVector();
	g_pLightPosVar = g_pPOMEffect->GetVariableByName( "LightPosition" )->AsVector();

	g_pColorTexVar = g_pPOMEffect->GetVariableByName( "ColorMap" )->AsShaderResource();
	g_pHeightTexVar = g_pPOMEffect->GetVariableByName( "NormalHeightMap" )->AsShaderResource();


	D3DX10_IMAGE_LOAD_INFO info;

	// manually specify format!!!
	//info.

    // Load the Textures
    hr = D3DX10CreateShaderResourceViewFromFile( pd3dDevice, L"rock_colormap.dds", NULL, NULL, &g_pColorTexRV, NULL );
	g_pColorTexVar->SetResource( g_pColorTexRV );

	hr = D3DX10CreateShaderResourceViewFromFile( pd3dDevice, L"rock_normalmap.dds", NULL, NULL, &g_pHeightTexRV, NULL );
	g_pHeightTexVar->SetResource( g_pHeightTexRV );


	// TODO: fixup the vertex layout and add the vertex information to the data!


    // Define the input layout
    D3D10_INPUT_ELEMENT_DESC layout[] =
    {
        { "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },  
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 }, 
		{ "NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 20, D3D10_INPUT_PER_VERTEX_DATA, 0 }, 
		{ "TANGENT", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 32, D3D10_INPUT_PER_VERTEX_DATA, 0 }, 
		{ "BINORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 44, D3D10_INPUT_PER_VERTEX_DATA, 0 }, 
    };
    UINT numElements = sizeof(layout) / sizeof(layout[0]);

    // Create the input layout
	D3D10_PASS_DESC PassDesc = {0};
    ID3D10EffectPass* pPass = g_pPOMTech->GetPassByIndex( 0 ); 
	pPass->GetDesc( &PassDesc );

    V_RETURN( pd3dDevice->CreateInputLayout( layout, numElements, PassDesc.pIAInputSignature, PassDesc.IAInputSignatureSize, &g_pVertexLayout ) );

    // Set the input layout
    pd3dDevice->IASetInputLayout( g_pVertexLayout );

    // Create vertex buffer
    SimpleVertex vertices[] =
    {
		// Top Face
        { D3DXVECTOR3( -1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ) },
        { D3DXVECTOR3(  1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ) },
        { D3DXVECTOR3(  1.0f,  1.0f,  1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ) },
        { D3DXVECTOR3( -1.0f,  1.0f,  1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ) },

		// Bottom Face
        { D3DXVECTOR3( -1.0f, -1.0f,  1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  -1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ) },
        { D3DXVECTOR3(  1.0f, -1.0f,  1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( 0.0f,  -1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ) },
        { D3DXVECTOR3(  1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( 0.0f,  -1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ) },
        { D3DXVECTOR3( -1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  -1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ) },

		// Left Face
        { D3DXVECTOR3( -1.0f, -1.0f,  1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f,  1.0f,  1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },

		// Right Face
        { D3DXVECTOR3( 1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( 1.0f, -1.0f,  1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( 1.0f,  1.0f,  1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( 1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },

		// Back Face
        { D3DXVECTOR3( -1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3(  1.0f, -1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3(  1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f,  1.0f, -1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  0.0f, -1.0f ), D3DXVECTOR3( 1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },

		// Front Face
        { D3DXVECTOR3(  1.0f, -1.0f, 1.0f ), D3DXVECTOR2( 0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f, -1.0f, 1.0f ), D3DXVECTOR2( 1.0f, 0.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3( -1.0f,  1.0f, 1.0f ), D3DXVECTOR2( 1.0f, 1.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
        { D3DXVECTOR3(  1.0f,  1.0f, 1.0f ), D3DXVECTOR2( 0.0f, 1.0f ), D3DXVECTOR3( 0.0f,  0.0f, 1.0f ), D3DXVECTOR3( -1.0f,  0.0f, 0.0f ), D3DXVECTOR3( 0.0f,  1.0f, 0.0f ) },
	};

    D3D10_BUFFER_DESC bd;
    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( SimpleVertex ) * ( 24 );
    bd.BindFlags = D3D10_BIND_VERTEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;
    D3D10_SUBRESOURCE_DATA InitData;
    InitData.pSysMem = vertices;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pVertexBuffer ) );

    // Set vertex buffer
    UINT stride = sizeof( SimpleVertex );
    UINT offset = 0;
    pd3dDevice->IASetVertexBuffers( 0, 1, &g_pVertexBuffer, &stride, &offset );

    // Create index buffer
    // Create vertex buffer
    DWORD indices[] =
    {
		// top
        3,1,0,
        2,1,3,

		// bottom
        7,5,4,
        6,5,7,

		// left
        11,9,8,
        10,9,11,

		// right
        15,13,12,
        14,13,15,

		// back
        19,17,16,
        18,17,19,

		// front
        23,21,20,
        22,21,23,
    };

    bd.Usage = D3D10_USAGE_DEFAULT;
    bd.ByteWidth = sizeof( DWORD ) * 36;
    bd.BindFlags = D3D10_BIND_INDEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;
    InitData.pSysMem = indices;
    V_RETURN( pd3dDevice->CreateBuffer( &bd, &InitData, &g_pIndexBuffer ) );

    // Set index buffer
    pd3dDevice->IASetIndexBuffer( g_pIndexBuffer, DXGI_FORMAT_R32_UINT, 0 );

    // Set primitive topology
    pd3dDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

    // Initialize the world matrices
    D3DXMatrixIdentity( &g_World );

	g_EyePosition = D3DXVECTOR4( 0.0f, 1.5f, -4.0f, 1.0f );
	g_LightPosition = D3DXVECTOR4( 5.0f, 15.0f, -16.0f, 1.0f );

    // Initialize the view matrix
    D3DXVECTOR3 Eye( 0.0f, 1.5f, -4.0f );
    D3DXVECTOR3 At( 0.0f, 0.0f, 0.0f );
    D3DXVECTOR3 Up( 0.0f, 1.0f, 0.0f );
    D3DXMatrixLookAtLH( &g_View, &Eye, &At, &Up );


    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
// Create and set the depth stencil texture if needed
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr = S_OK;

    V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBufferSurfaceDesc ) );
    V_RETURN( g_D3DSettingsDlg.OnD3D10ResizedSwapChain( pd3dDevice, pBufferSurfaceDesc ) );

    g_HUD.SetLocation( pBufferSurfaceDesc->Width-170, 0 );
    g_HUD.SetSize( 170, 170 );

    // Setup the projection parameters again
    float fAspect = static_cast<float>( pBufferSurfaceDesc->Width )/static_cast<float>( pBufferSurfaceDesc->Height );
    D3DXMatrixPerspectiveFovLH( &g_Proj, D3DX_PI * 0.25f, fAspect, 0.1f, 100.0f );

	// Record the back buffer and depth buffer views
	pd3dDevice->OMGetRenderTargets( 1, &g_pBackBufferView, &g_pBackBufferDepthView );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	static double TotalTime = 0.0;
	static int FrameCount = 0;

	FrameCount++;

	if ( FrameCount > 1000 )
		TotalTime += fTime;

	if ( FrameCount == 2000 )
		int i = 0;

    // If the settings dialog is being shown, then
    // render it instead of rendering the app's scene
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.OnRender( fElapsedTime );
        return;
    }

	
	float ClearColor[4] = { 0.6f, 0.6f, 0.8f, 0.0f };

	g_pWorldVar->SetMatrix( (float*)&g_World );
	g_pWorldViewProjVar->SetMatrix( (float*)&g_WorldViewProj );
	g_pEyePosVar->SetFloatVector( (float*)&g_EyePosition ); 
	g_pLightPosVar->SetFloatVector( (float*)&g_LightPosition );

	g_pColorTexVar->SetResource( g_pColorTexRV );
	g_pHeightTexVar->SetResource( g_pHeightTexRV );

	pd3dDevice->ClearRenderTargetView( g_pBackBufferView, ClearColor );
    pd3dDevice->ClearDepthStencilView( g_pBackBufferDepthView, D3D10_CLEAR_DEPTH, 1.0, 0 );


    D3D10_TECHNIQUE_DESC techDesc;
    g_pPOMTech->GetDesc( &techDesc );
    for( UINT p = 0; p < techDesc.Passes; ++p )
    {
        g_pPOMTech->GetPassByIndex( p )->Apply(0);
        pd3dDevice->DrawIndexed( 36, 0, 0 );
    }

    //
    // Render the UI
    //
    g_HUD.OnRender( fElapsedTime ); 
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10ReleasingSwapChain();
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D10DestroyDevice();
    g_D3DSettingsDlg.OnD3D10DestroyDevice();
    DXUTGetGlobalResourceCache().OnDestroyDevice();

    SAFE_RELEASE( g_pVertexBuffer );
    SAFE_RELEASE( g_pIndexBuffer );
    SAFE_RELEASE( g_pVertexLayout );

	SAFE_RELEASE( g_pPOMEffect );

    SAFE_RELEASE( g_pColorTexRV );
    SAFE_RELEASE( g_pHeightTexRV );

	SAFE_RELEASE( g_pBackBufferView );
	SAFE_RELEASE( g_pBackBufferDepthView );
}


//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
    // Rotate cube around the origin
	D3DXMATRIX Local;

	D3DXMatrixRotationX( &Local, 7.0f*DEG2RAD((float)fTime) );
	D3DXMatrixRotationY( &g_World, 6.0f*DEG2RAD((float)fTime) );

	D3DXMatrixMultiply( &g_World, &g_World, &Local );
	D3DXMatrixMultiply( &g_WorldView, &g_World, &g_View );
	D3DXMatrixMultiply( &g_WorldViewProj, &g_WorldView, &g_Proj );
}

//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing, void* pUserContext )
{
    // Pass messages to dialog resource manager calls so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;


    // Pass messages to settings dialog if its active
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;



    return 0;
}


//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( bKeyDown )
    {
        switch( nChar )
        {
            case VK_F1: // Change as needed                
				g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() );
				break;
        }
    }
}



//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{    
    switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN: DXUTToggleFullScreen(); break;
        case IDC_TOGGLEREF:        DXUTToggleREF(); break;
        case IDC_CHANGEDEVICE:     g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() ); break;

    }    
}

